<?php

/**
 * Ebizmarts_MAgeMonkey Magento JS component
 *
 * @category    Ebizmarts
 * @package     Ebizmarts_MageMonkey
 * @author      Ebizmarts Team <info@ebizmarts.com>
 * @copyright   Ebizmarts (http://ebizmarts.com)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

namespace Ebizmarts\MageMonkey\Model\Plugin;

class Subscriber {

    /**
     * @var \Ebizmarts\MageMonkey\Helper\Data
     */
    protected $_helper;

    /**
     * @var \Magento\Customer\Model\Customer
     */
    protected $_customer;

    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $_customerSession;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @param \Ebizmarts\MageMonkey\Helper\Data $helper
     * @param \Magento\Customer\Model\Customer $customer
     * @param \Magento\Customer\Model\Session $customerSession
     */
    protected $_api = null;

    public function __construct(
    \Ebizmarts\MageMonkey\Helper\Data $helper, \Magento\Customer\Model\Customer $customer, \Magento\Customer\Model\Session $customerSession, \Magento\Store\Model\StoreManagerInterface $storeManager, \Ebizmarts\MageMonkey\Model\Api $api
    ) {
        $this->_helper = $helper;
        $this->_customer = $customer;
        $this->_customerSession = $customerSession;
        $this->_storeManager = $storeManager;
        $this->_api = $api;
    }

    public function beforeUnsubscribeCustomerById(
    $subscriber, $customerId
    ) {
        $subscriber->loadByCustomerId($customerId);
        if ($subscriber->getMagemonkeyId()) {
            //$api = $this->_api->loadByStore($subscriber->getStoreId());
            $this->_api->listDeleteMember($this->_helper->getDefaultList(), $subscriber->getMagemonkeyId());
            $subscriber->setMagemonkeyId('');
        }
        return array($customerId);
    }

    public function beforeSubscribeCustomerById(
    $subscriber, $customerId
    ) {

        
        $customer = $this->_customer->load($customerId);
        $storeId = $customer->getStoreId();
        $email = $customer->getEmail();

        $subscriber->loadByEmail($email);
        
//        $isSubscribeOwnEmail = $this->_customerSession->isLoggedIn()
//            && $this->_customerSession->getCustomerDataObject()->getEmail() == $email;
//        
//        if (!$this->_helper->isDoubleOptInEnabled($storeId) || $isSubscribeOwnEmail) {

            if ($this->_helper->isMonkeyEnabled($storeId)) {

                $api = $this->_api;

                $status = 'subscribed';

                $mergeVars = $this->_helper->getMergeVars($subscriber,true,false);

                if ($mergeVars) {
                    $data = array('list_id' => $this->_helper->getDefaultList(), 'email_address' => $email, 'email_type' => 'html', 'status' => $status, 'merge_fields' => $mergeVars);
                } else {
                    $data = array('list_id' => $this->_helper->getDefaultList(), 'email_address' => $email, 'email_type' => 'html', 'status' => $status);
                }

                try {
                    if ($subscriber->getMagemonkeyId()) {

                        $return = $api->listUpdateMember($this->_helper->getDefaultList(), json_encode($data), $subscriber->getMagemonkeyId());
                    } else {
                        $return = $api->listCreateMember($this->_helper->getDefaultList(), json_encode($data));
                        if (isset($return->id)) {
                            $subscriber->setMagemonkeyId($return->id);
                        }
                    }
                } catch (\Exception $e) {

                    $this->_helper->log($e->getMessage());
                    if (stripos($e->getMessage(), ' Status: 400 ') === false) {
                        throw new \Exception($e->getMessage());
                    }
                }
            }
//        }
        return array($customerId);
    }

    /**
     * Confirms subscriber newsletter
     *
     * @param string $code
     * @return boolean
     */
    public function beforeConfirm($subscriber, $code) {


        if ($subscriber->getCode() == $code) {

            $email = $subscriber->getEmail();
            $storeId = $this->_storeManager->getStore()->getId();

            if ($this->_helper->isMonkeyEnabled($storeId)) {

                $api = $this->_api;

                $status = 'subscribed';

                $mergeVars = $this->_helper->getMergeVars($subscriber,false,true);

                if ($mergeVars) {
                    $data = array('list_id' => $this->_helper->getDefaultList(), 'email_address' => $email, 'email_type' => 'html', 'status' => $status, 'merge_fields' => $mergeVars);
                } else {
                    $data = array('list_id' => $this->_helper->getDefaultList(), 'email_address' => $email, 'email_type' => 'html', 'status' => $status);
                }
                try {

                    if ($subscriber->getMagemonkeyId()) {
                        $return = $api->listUpdateMember($this->_helper->getDefaultList(), json_encode($data), $subscriber->getMagemonkeyId());
                    } else {

                        $return = $api->listCreateMember($this->_helper->getDefaultList(), json_encode($data));
                        if (isset($return->id)) {
                            $subscriber->setMagemonkeyId($return->id);
                        }
                    }
                } catch (\Exception $e) {
                    $this->_helper->log($e->getMessage());
                    if (stripos($e->getMessage(), ' Status: 400 ') === false) {

                        throw new \Exception($e->getMessage());
                    }
                }
            }
        }

        return array($code);
    }

    public function beforeSubscribe(
    $subscriber, $email
    ) {
//        $obj = \Magento\Framework\App\ObjectManager::getInstance();
//        $subscriber->addData($obj->create('Magento\Newsletter\Model\ResourceModel\Subscriber')->loadByEmail($email));

        $storeId = $this->_storeManager->getStore()->getId();

        $isSubscribeOwnEmail = $this->_customerSession->isLoggedIn()
            && $this->_customerSession->getCustomerDataObject()->getEmail() == $email;
        
        if (!$this->_helper->isDoubleOptInEnabled($storeId) || $isSubscribeOwnEmail) {

            if ($this->_helper->isMonkeyEnabled($storeId)) {

                $api = $this->_api;

                $status = 'subscribed';

                $mergeVars = $this->_helper->getMergeVars($subscriber,false,false);

                if ($mergeVars) {
                    $data = array('list_id' => $this->_helper->getDefaultList(), 'email_address' => $email, 'email_type' => 'html', 'status' => $status, 'merge_fields' => $mergeVars);
                } else {
                    $data = array('list_id' => $this->_helper->getDefaultList(), 'email_address' => $email, 'email_type' => 'html', 'status' => $status);
                }

                try {
                    if ($subscriber->getMagemonkeyId()) {
                        $return = $api->listUpdateMember($this->_helper->getDefaultList(), json_encode($data), $subscriber->getMagemonkeyId());
                    } else {
                        $return = $api->listCreateMember($this->_helper->getDefaultList(), json_encode($data));
                        if (isset($return->id)) {
                            $subscriber->setMagemonkeyId($return->id);
                        }
                    }
                } catch (\Exception $e) {
                    $this->_helper->log($e->getMessage());
                    if (stripos($e->getMessage(), ' Status: 400 ') === false) {
                        throw new \Exception($e->getMessage());
                    }
                }
            }
        }
        return array($email);
    }

    public function beforeUnsubscribe(
    $subscriber
    ) {
        if ($subscriber->getMagemonkeyId()) {
            $this->_api->listDeleteMember($this->_helper->getDefaultList(), $subscriber->getMagemonkeyId());
            $subscriber->setMagemonkeyId('');
        }
        return null;
    }

}
